/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#define MAXPROFILESTR 256
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <string.h>
#include <stdio.h>
#include "netbox.h"
#include "strbox.h"
#include "berutil.h"
#include "zdefs.h"
#include "init.h"
#include "search.h"
#include "present.h"
#include "zdecode.h"
#include "zencode.h"
#include "query.h"
#include "log.h"
#include "rpn.h"
#include "zutils.h"
#include "freemem.h"
#include "ztags.h"
#include "init.h"

/*
FUNCTION:	fwInitRequest()
PURPOSE:	Sends an InitRequest over the wire.
PRE:		fp = log file descriptor.
		s = socket descriptor.
		pdu = valid InitRequest PDU structure.
POST:		On success, returns 1 and the PDU is sent.
		On failure, returns an error code and the PDU is not sent.
ERRORS:		See zEncodePDU()
*/
int fwInitRequest(FILE *fp, NETBOXPROFILE*  s, PDU *pdu)
{
 	int result;
 
	result=encodeZ3950PDU(pdu,s);
	if(result!=1) {
		fwLog(fp,0,0,"init:zInitRequest():failed with %i",result);
		return result;
	}

	return 1;
}

/*
FUNCTION:	fwInitResponse()
PURPOSE:	Sends an InitResponse over the wire.
PRE:		fp = log file descriptor.
		s = socket descriptor.
		pdu = valid InitResponse PDU structure.
POST:		On success, returns 1 and the PDU is sent.
		On failure, returns an error code and the PDU is not sent.
ERRORS:		See zEncodePDU()
*/
int fwInitResponse(FILE *fp, NETBOXPROFILE* s, PDU *pdu) 
{
	int result;
 
	result=encodeZ3950PDU(pdu,s);
	if(result!=1) {
		fwLog(fp,0,0,"init:zInitResponse():failed with %i",result);
		return result;
	}
	
	return 1;
}

/*
FUNCTION:	fwBuildInitRequest()
PURPOSE:	Allocates an InitRequest PDU and fills with specified
		parameters.
PRE:		pIdAuthentication = authentication string.
		pImplementationId = implementation identification.
		pImplementationName = implementation name.
		pImplementationVersion = implementation version.
		pProtocolVersion = Z39.50 protocol version.
		pOptions = Z39.50 options supported.
		pPreferredMsgSize = preferred buffer size.
		pMaxRecordSize = maximum buffer size.
		pUserInfoField = misc user information.
		pReferenceId= instance reference identification.
POST:		On success, returns an InitRequest PDU.
		On failure, returns NULL PDU pointer.
*/
PDU *fwBuildInitRequest(CHAR *pIdAuthentication, CHAR *pImplementationId,
			CHAR *pImplementationName,
			CHAR *pImplementationVersion,
			CHAR *pProtocolVersion, CHAR *pOptions,
			long pPreferredMsgSize, long pMaxRecordSize,
			CHAR *pUserInfoField, CHAR *pReferenceId)
{
	PPDU pdu;
 
	if((pdu = (PPDU)calloc(1,sizeof(PDU))) == NULL)
		return NULL;
	pdu->which = t_initRequest;
	pdu->u.initRequest=
		(PINITIALIZEREQUEST)calloc(1,sizeof(INITIALIZEREQUEST));

        if(*pIdAuthentication!=NULL) {
		pdu->u.initRequest->idAuthentication=(PANY)calloc(1,sizeof(ANY));
		pdu->u.initRequest->idAuthentication->PETag=zIdAuthenticationTag;
		pdu->u.initRequest->idAuthentication->PEForm=ZNOTCONSTRUCTED;
		pdu->u.initRequest->idAuthentication->u.data=
		encodeOctetStr(strdup(pIdAuthentication));
	} else
		pdu->u.initRequest->idAuthentication=NULL;

	if(*pImplementationId!=NULL)
		pdu->u.initRequest->implementationId =
			encodeOctetStr(strdup(pImplementationId));
	else
		pdu->u.initRequest->implementationId = NULL;

	if(*pImplementationName!=NULL)
		pdu->u.initRequest->implementationName = 
			encodeOctetStr(strdup(pImplementationName));
	else
		pdu->u.initRequest->implementationName = NULL;

	if(*pImplementationVersion!=NULL)
		pdu->u.initRequest->implementationVersion = 
			encodeOctetStr(strdup(pImplementationVersion));

	else
		pdu->u.initRequest->implementationVersion = NULL;

	pdu->u.initRequest->protocolVersion =
		encodeOctetStr(strdup(pProtocolVersion));
	pdu->u.initRequest->options = encodeOctetStr(strdup(pOptions));
	pdu->u.initRequest->preferredMessageSize = pPreferredMsgSize;
	pdu->u.initRequest->maximumRecordSize = pMaxRecordSize; 

        if(*pUserInfoField!=NULL) {
		pdu->u.initRequest->userInformationField = 
			(PPRESENTATIONELEMENT)calloc(1,sizeof(PRESENTATIONELEMENT));
		pdu->u.initRequest->userInformationField->u.data = 
			encodeOctetStr(strdup(pUserInfoField));
		pdu->u.initRequest->userInformationField->PETag=zUserInfoFieldTag;
		pdu->u.initRequest->userInformationField->PEForm=ZNOTCONSTRUCTED;
	} else pdu->u.initRequest->userInformationField = NULL;

	if(*pReferenceId!=NULL)
		pdu->u.initRequest->referenceId =
			encodeOctetStr(strdup(pReferenceId));
	else
		pdu->u.initRequest->referenceId = NULL;
 
	return pdu;
}

/*
FUNCTION:	fwBuildInitResponse()
PURPOSE:	Allocates an InitResponse PDU and fills with specified
		parameters.
PRE:		pImplementationId = implementation identification.
		pImplementationName = implementation name.
		pImplementationVersion = implementation version.
		pProtocolVersion = Z39.50 protocol version.
		pOptions = Z39.50 options supported.
		pPreferredMsgSize = preferred buffer size.
		pMaxRecordSize = maximum buffer size.
		pUserInfoField = misc user information.
		pReferenceId= instance reference identification.
		pResult = result of the initialization.
POST:		On success, returns an InitRequest PDU.
		On failure, returns NULL PDU pointer.
*/
PDU *fwBuildInitResponse(CHAR *pImplementationId,
		       CHAR *pImplementationName, CHAR *pImplementationVersion,
		       CHAR *pProtocolVersion, CHAR *pOptions,
                       long pPreferredMsgSize, long pMaxRecordSize,
		       CHAR *pUserInfoField, CHAR *pReferenceId,
                       unsigned char pResult) 
{
	PPDU pdu;

	pdu = (PPDU)calloc(1,sizeof(PDU));
	pdu->which = t_initResponse;
	pdu->u.initResponse=
		(PINITIALIZERESPONSE)calloc(1,sizeof(INITIALIZERESPONSE));
	pdu->u.initResponse->protocolVersion =
		encodeOctetStr(strdup(pProtocolVersion));
	pdu->u.initResponse->options = 
		encodeOctetStr(strdup(pOptions));
	pdu->u.initResponse->preferredMessageSize = 
		pPreferredMsgSize; 
	pdu->u.initResponse->maximumRecordSize = 
		pMaxRecordSize; 
	pdu->u.initResponse->implementationId = 
		encodeOctetStr(strdup(pImplementationId));
	pdu->u.initResponse->implementationName = 
		encodeOctetStr(strdup(pImplementationName));
	pdu->u.initResponse->implementationVersion = 
		encodeOctetStr(strdup(pImplementationVersion));
	pdu->u.initResponse->userInformationField = 
		(PPRESENTATIONELEMENT)calloc(1,sizeof(PRESENTATIONELEMENT));
	pdu->u.initResponse->userInformationField->u.data = 
		encodeOctetStr(strdup(pUserInfoField));
	pdu->u.initResponse->userInformationField->PETag=zUserInfoFieldTag;
	pdu->u.initResponse->userInformationField->PEForm=ZNOTCONSTRUCTED;
	pdu->u.initResponse->referenceId = 
		encodeOctetStr(strdup(pReferenceId));
	pdu->u.initResponse->result=(unsigned char)pResult;

	return pdu;
}

/*
FUNCTION:	fwFreeInitRequest()
PURPOSE:	Frees an allocated InitRequest PDU structure.
PRE:		pdu = pointer to allocated PDU structure.
POST:		Memory allocated by pdu is freed.
*/
void fwFreeInitRequest(PPDU pdu) 
{

	freeAny(pdu->u.initRequest->idAuthentication);
	freeOctetString(pdu->u.initRequest->implementationId);
	freeOctetString(pdu->u.initRequest->implementationName);
	freeOctetString(pdu->u.initRequest->implementationVersion);
	freeBitString(pdu->u.initRequest->protocolVersion);
	freeBitString(pdu->u.initRequest->options);
	freeExternal(pdu->u.initRequest->userInformationField);
	freeOctetString(pdu->u.initRequest->referenceId);
	free(pdu->u.initRequest);
	free(pdu);
}

/*
FUNCTION:	fwFreeInitResponse()
PURPOSE:	Frees an allocated InitResponse PDU structure.
PRE:		pdu = pointer to allocated PDU structure.
POST:		Memory allocated by pdu is freed.
*/
void fwFreeInitResponse(PPDU pdu) 
{

	freeOctetString(pdu->u.initResponse->referenceId);
	freeBitString(pdu->u.initResponse->protocolVersion);
	freeBitString(pdu->u.initResponse->options);
	freeOctetString(pdu->u.initResponse->implementationId);
	freeOctetString(pdu->u.initResponse->implementationName);
	freeOctetString(pdu->u.initResponse->implementationVersion);
	freeExternal(pdu->u.initResponse->userInformationField);
	free(pdu->u.initResponse);
	free(pdu);
}

