/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#include "platform.h"
#ifdef PLATFORM_MSDOS
#ifdef PLATFORM_BORLANDC
#include <alloc.h>
#endif
#ifdef PLATFORM_MSC
#include <malloc.h>
#endif
#endif
#include <stdio.h>
#include <string.h>
#include "netbox.h"
#include "berutil.h"
#include "zdefs.h"
#include "strbox.h"
#include "zutils.h"
#include "ztags.h"
#include "ztypes.h"
#include "zrecords.h"
#include "records.h"

/* Local functions */
PPRESENTATIONELEMENT findTag(PPRESENTATIONELEMENT pe, long tag);
PPRESENTATIONELEMENT findLastTag(PPRESENTATIONELEMENT pe, long *tag);

/*
FUNCTION:	numRecords()
PURPOSE:	Determines number of records held in RECORDS structure.
PRE:		r = valid RECORDS structure.
POST:		returns number of records in the structure.
*/
long numRecords(PRECORDS r) {
	long count=0;

	if(r->which==t_dataBaseOrSurDiagnostics) {
		PNAMEPLUSRECORDLIST nprList;
		nprList=r->u.dataBaseOrSurDiagnostics;
		while(nprList!=NULL) {
			count++;
                        nprList=nprList->next;
		}
	} else {
        	/* contains 1 nonSurrogateDiagnostic record */
		count = 1;
        }

	return count;	
}

/*
FUNCTION:	isDiagnostic()
PURPOSE:	Determines whether the RECORDS structure has a diagnostic
		record instead of database or surrogate diagnostic records.
PRE:		r = valid RECORDS structure.
POST:		returns 1 if there is a diagnostic record.
		returns 0 if there is not a diagnostic record.
*/
int isDiagnostic(PRECORDS r) {
	if(r->which==t_nonSurrogateDiagnostic) {
		return 1;
	}
	return 0;
}

/*
FUNCTION:	recLength()
PURPOSE:	Determines the length of a specified record in a RECORDS
		structure.
PRE:		r = valid RECORDS structure.
POST:		On success, returns the length of the record.
		On failure, returns -1 if:
			recNum > numRecords(r)
			recNum < 0
			r only contains a nonsurrogatediagnostic record

*/
long recLength(PRECORDS r, long recNum) {
	PNAMEPLUSRECORDLIST nprList;
	long count=0;
	long numRecs;

	numRecs = numRecords(r);

	if((recNum > numRecs) || (recNum < 0))
		return -1;
	if(isDiagnostic(r)) {
                /* Not interested */
		return -1;
        }

	nprList=r->u.dataBaseOrSurDiagnostics;
	while(count<recNum) {
		count++;
		nprList=nprList->next;
	}
	/* nprList now points to the correct record */
	if(nprList->item->recordOrSurrogate.which==t_databaseRecord) {
		/* Now, dig through the oids, tags and junk to get to
		 * the record data.
		 */
		long tag;
		PPRESENTATIONELEMENT pe;
		PPRESENTATIONELEMENTLIST peList;

		pe=nprList->item->recordOrSurrogate.u.databaseRecord;
		pe=findLastTag(pe,&tag);
 		return(pe->u.data->length);

	} else {
                /* Not interested */
		return -1;
        }

}

/*
FUNCTION:	findTag()
PURPOSE:	Searches through a PRESENTATIONELEMENT for the first
		occurrance of a specified tag value, returning a pointer
		to the PRESENTATIONELEMENT with that tag.
PRE:		pe = a valid PRESENTATIONELEMENT structure.
		tag = ASN.1 tag
POST:		On success, returns a pointer to the presentationElement.
		On failure, returns NULL.
*/
PPRESENTATIONELEMENT findTag(PPRESENTATIONELEMENT pe, long tag) {
	PPRESENTATIONELEMENT pePtr;

	if(pe->PETag == tag)
		return pe;
	if(pe->PEForm > 0) {
		PPRESENTATIONELEMENTLIST peListPtr;
		peListPtr=pe->u.list;
		while(peListPtr) {
			pePtr=findTag(peListPtr->item,tag);
                	peListPtr=peListPtr->next;
		}
	} else
		return NULL;

	return pePtr;
}

/*
FUNCTION:	findLastTag()
PURPOSE:	Searches through a PRESENTATIONELEMENT for the last
		occurrance of any tag, returning a pointer to the
		PRESENTATIONELEMENT with that tag.
PRE:		pe = a valid PRESENTATIONELEMENT structure.
		tag = place holder where the tag value will be stored.
POST:		Returns a pointer to the last tagged presentationElement.
NOTES:		This is not the best solution, but all records I know of
		so far are stored as the last element in a presentation
		element, so I use this function to find the record.
*/
PPRESENTATIONELEMENT findLastTag(PPRESENTATIONELEMENT pe, long *tag) {
	PPRESENTATIONELEMENT pePtr;
	PPRESENTATIONELEMENTLIST pePrevPtr;

	if(pe->PEForm > 0) {
		PPRESENTATIONELEMENTLIST peListPtr;
		peListPtr=pe->u.list;
		while(peListPtr!=NULL) {
			pePtr=findLastTag(peListPtr->item,tag);
                        peListPtr=peListPtr->next;
		}
		return pePtr;
	} 

	return pe;
}

/*
FUNCTION:	findRecord()
PURPOSE:	Searches through a RECORDS structure for a specified
		record number.
PRE:		r = a valid RECORDS structure.
		0 <= recNum < numRecords(r) 
POST:		On success,  returns a pointer to the record data.
		On failure, returns NULL.
NOTES:		This won't work in all cases, but does the job for
		now.  To obtain the length of the record, call
                recLength().
*/
char *findRecord(PRECORDS r, long recNum) {
	PNAMEPLUSRECORDLIST nprList;
	long count=0L;

	nprList=r->u.dataBaseOrSurDiagnostics;
	while(count < recNum) {
		count++;
		nprList=nprList->next;
	}
	/* nprList now points to the correct record */
	if(nprList->item->recordOrSurrogate.which==t_databaseRecord) {
		/* Now, dig through the oids, tags and junk to get to
		 * the record data.
		 */
                long tag;
		PPRESENTATIONELEMENT pe;
		PPRESENTATIONELEMENTLIST peList;

		pe=nprList->item->recordOrSurrogate.u.databaseRecord;
		pe=findLastTag(pe,&tag);
 		return((char *)pe->u.data->data);

	} else {
        	/* Diagnostic record...not interested */
		return NULL;

        }

}

/*
FUNCTION:	copyRecord()
PURPOSE:	Copies the specified record into a buffer
PRE:		r = a valid RECORDS structure.
		0 <= recNum < numRecords(r)
                buf = allocated character buffer.
		len = maximum length for buf.
POST:		On success, returns the number of bytes copied.
		On failure, returns -1.
*/
long copyRecord(PRECORDS r, long recNum, long start, char *buf, long len) {
	long length,lenToUse;
	char *bufPtr;
	                 	
	if((length = recLength(r,recNum)) == -1)
		return(-1);
	if((bufPtr = findRecord(r,recNum)) == NULL)
		return(-1);
	if(len < length)
		length=len;

	bufPtr+=start;
	if(len > (length-start))
		lenToUse = (length-start);
	else
		lenToUse = len;

	memcpy(buf, bufPtr, lenToUse);

	return(lenToUse);

}
