/***************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery
and Retrieval, 1994.

Permission to use, copy, modify, distribute, and sell this software
and its documentation, in whole or in part, for any purpose is hereby
granted without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included
in this distribution must remain intact.
2. Users of this software agree to make their best efforts (a) to
return to MCNC any improvements or extensions that they make, so that
these may be included in future releases; and (b) to inform MCNC/CNIDR
of noteworthy uses of this software.

3. The names of MCNC and Clearinghouse for Networked Information
Discovery and Retrieval may not be used in any advertising or publicity
relating to the software without the specific, prior written permission
of MCNC/CNIDR.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

***************************************************************/
#define PLATFORM_INC_MALLOC
#include "platform.h"
#include <string.h>
#include <stdio.h>
#include "netbox.h"
#include "strbox.h"
#include "berutil.h"
#include "zdefs.h"
#include "init.h"
#include "search.h"
#include "present.h"
#include "zdecode.h"
#include "zencode.h"
#include "query.h"
#include "log.h"
#include "rpn.h"
#include "zutils.h"
#include "freemem.h"
#include "ztags.h"
#include "access.h"

#define NEW(a) (a*)calloc(1,sizeof(a))
/*
FUNCTION:	fwDeleteResultSetRequest()
PURPOSE:	Sends a DeleteResultSetRequest over the wire.
PRE:		fp = log file descriptor.
		s = socket descriptor.
		pdu = valid DeleteResultSetRequest PDU structure.
POST:		On success, returns 1 and the PDU is sent.
		On failure, returns an error code and the PDU is not sent.
ERRORS:		See zEncodePDU()
*/
int fwDeleteResultSetRequest(FILE *fp, NETBOXPROFILE*  s, PDU *pdu)
{
 	int result;
 
	result=encodeZ3950PDU(pdu,s);
	if(result!=1) {
		fwLog(fp,0,0,"result:fwDeleteResultSetRequest():failed with %i",result);
		return result;
	}

	return 1;
}

/*
FUNCTION:	fwDeleteResultSetResponse()
PURPOSE:	Sends a DeleteResultSetResponse over the wire.
PRE:		fp = log file descriptor.
		s = socket descriptor.
		pdu = valid DeleteResultSetResponse PDU structure.
POST:		On success, returns 1 and the PDU is sent.
		On failure, returns an error code and the PDU is not sent.
ERRORS:		See zEncodePDU()
*/
int fwDeleteResultSetResponse(FILE *fp, NETBOXPROFILE* s, PDU *pdu)
{
	int result;
 
	result=encodeZ3950PDU(pdu,s);
	if(result!=1) {
		fwLog(fp,0,0,"result:fwDeleteResultSetResponse():failed with %i",result);
		return result;
	}
	
	return 1;
}

/*
FUNCTION:	fwBuildDeleteResultSetRequest()
PURPOSE:	Allocates a DeleteResultSetRequest PDU and fills with specified
		parameters.
PRE:		refId = optional reference Id
		deleteOperation = Operation defined by standard
		resultSetList = optional list of result sets,
		comma-deliminated.
POST:		On success, returns a DeleteResultSetRequest PDU.
		On failure, returns NULL PDU pointer.
NOTES:		Only takes 1 result set now.  Should be able to take
		a comma-deliminated list of them and build a
                RESULTSETLIST from that.
*/
PDU *fwBuildDeleteResultSetRequest(char *refId, long deleteOperation,
				char *resultSetList)
{
	PPDU pdu;
 
	if((pdu = NEW(PDU)) == NULL)
		return NULL;
	pdu->which = t_deleteResultSetRequest;
	if((pdu->u.deleteResultSetRequest=NEW(DELETERESULTSETREQUEST))==NULL) {
        	free(pdu);
        	return NULL;
	}
	if(*refId!=NULL)
		pdu->u.deleteResultSetRequest->referenceId=
			encodeOctetStr(strdup(refId));
	pdu->u.deleteResultSetRequest->deleteOperation.operation=deleteOperation;
        /* Only support a single result set name now */
	if(*resultSetList != NULL) {
		pdu->u.deleteResultSetRequest->resultSetList=
			NEW(RESULTSETLIST);
		pdu->u.deleteResultSetRequest->resultSetList->item=
		encodeOctetStr(strdup(resultSetList));
		pdu->u.deleteResultSetRequest->resultSetList->next=NULL;
	}
	return pdu;
}

/*
FUNCTION:	fwFreeDeleteResultSetRequest()
PURPOSE:	Frees an allocated DeleteResultSetRequest PDU structure.
PRE:		pdu = pointer to allocated PDU structure.
POST:		Memory allocated by pdu is freed.
*/
void fwFreeDeleteResultSetRequest(PPDU pdu) 
{
	PRESULTSETLIST list;
	freeOctetString(pdu->u.deleteResultSetRequest->referenceId);
	if(pdu->u.deleteResultSetRequest->resultSetList != NULL) {
		list=pdu->u.deleteResultSetRequest->resultSetList;
		while(list!=NULL) {
			freeOctetString(pdu->u.deleteResultSetRequest->resultSetList->item);
			list = list->next;
		}
		free(pdu->u.deleteResultSetRequest->resultSetList);
        }
	free(pdu->u.deleteResultSetRequest);
	free(pdu);
}
